//-------------------------------------------------------------------------
#include	<avr/io.h>
#include 	<string.h>
#include	"mmc.h"
#include	"spi.h"
//-------------------------------------------------------------------------
#define	CS			2  //PB1
#define	CSH			PORTB.OUT|=0x02
#define	CSL			PORTB.OUT&=~0x02
struct sd_raw_info  cardinf;
uint8_t sd_raw_card_type;
//-------------------------------------------------------------------------
void mmcInit(void){
	spiInit();						// initialize SPI interface	
	PORTB.DIR|=0x02;				// release chip select
	CSH;
}
//-------------------------------------------------------------------------
unsigned char Card_IS_In(void)
{
if(PORTC.IN&(1<<1))return 0;
	return 1;
}
//-------------------------------------------------------------------------
uint8_t mmcReset_1(void)
{
	uint8_t i,retry,r1,response;
	uint16_t j ;
	uint32_t arg;
	SPI_Low();
	
	retry=0;
	do{
		for(i=0;i<0x10;i++) spiTransferByte(0xFF);	// send dummy at CS high		
		r1=mmcSendCommand(MMC_GO_IDLE_STATE, 0);	// resetting card
		retry++;
		if(retry>50) return -1;
	} while(r1 != MMC_R1_IDLE_STATE);

#if SD_RAW_SDHC
    /* check for version of SD card specification */
    response = mmcSendCommand(CMD_SEND_IF_COND, 0x100 /* 2.7V - 3.6V */ | 0xaa /* test pattern */);
    if((response & (1 << R1_ILL_COMMAND)) == 0)
    {
		CSL;
        spiTransferByte(0xFF);
        spiTransferByte(0xFF);
        if((spiTransferByte(0xFF) & 0x01) == 0){
			CSH;
            return -1; /* card operation voltage range doesn't match */
			}
        if(spiTransferByte(0xFF) != 0xaa){
			CSH;
            return -1; /* wrong test pattern */
			}
		CSH;
        /* card conforms to SD 2 card specification */
        sd_raw_card_type |= (1 << SD_RAW_SPEC_2);
    }
    else
#endif
    {
        /* determine SD/MMC card type */
        mmcSendCommand(CMD_APP, 0);
        response = mmcSendCommand(CMD_SD_SEND_OP_COND, 0);
        if((response & (1 << R1_ILL_COMMAND)) == 0)
        {
            /* card conforms to SD 1 card specification */
            sd_raw_card_type |= (1 << SD_RAW_SPEC_1);
        }
        else
        {
            /* MMC card */
        }
    }
    /* wait for card to get ready */
    for( j = 0; ; ++j)
    {
        if(sd_raw_card_type & ((1 << SD_RAW_SPEC_1) | (1 << SD_RAW_SPEC_2)))
        {
            arg = 0;
#if SD_RAW_SDHC
            if(sd_raw_card_type & (1 << SD_RAW_SPEC_2))
                arg = 0x40000000;
#endif
            mmcSendCommand(CMD_APP, 0);
            response = mmcSendCommand(CMD_SD_SEND_OP_COND, arg);
        }
        else
        {
            response = mmcSendCommand(CMD_SEND_OP_COND, 0);
        }
		
        if((response & (1 << R1_IDLE_STATE)) == 0)
            break;

        if(j == 0x7fff)
        {
            CSH;
            return -1;
        }
    }
	CSL;
#if SD_RAW_SDHC
    if(sd_raw_card_type & (1 << SD_RAW_SPEC_2))
    {
        if(mmcSendCommand(CMD_READ_OCR, 0))
        {
            CSH;
            return -1;
        }

        if(spiTransferByte(0xFF) & 0x40)
            sd_raw_card_type |= (1 << SD_RAW_SPEC_SDHC);

        spiTransferByte(0xFF);
        spiTransferByte(0xFF);
        spiTransferByte(0xFF);
    }
#endif
	CSH;
    /* set block size to 512 bytes */
    if(mmcSendCommand(MMC_SET_BLOCKLEN, 512))
    {
        CSH;
        return -1;
    }
	mmcSendCommand(MMC_CRC_ON_OFF, 0);			// turn off CRC checking
	SPI_High();

	return 0;
}
//-------------------------------------------------------------------------
uint8_t mmcReset(void){
	uint8_t i;
	for(i=0;i<2;i++)if(mmcReset_1()==0){return 0;}
	return -1;
}
//-------------------------------------------------------------------------
uint8_t mmcSendCommand(uint8_t cmd, uint32_t arg){
	uint8_t r1;

	CSL;											// assert chip select	
	r1 = mmcCommand(cmd, arg);						// issue the command
	CSH;											// release chip select
	return r1;
}
//-------------------------------------------------------------------------
uint8_t mmcRead(uint32_t sector, uint8_t* buffer ){
	uint8_t r1;
	uint16_t i;

	CSL;											// assert chip select	
	r1 = mmcCommand(MMC_READ_SINGLE_BLOCK, sector<<9);	// issue command
	if(r1 != 0)return r1;	
	while(spiTransferByte(0xFF)!=MMC_STARTBLOCK_READ);// wait for block start
	
	for(i=0; i<0x200; i++){
		*buffer++ = spiTransferByte(0xFF);			// read in data
	}
	
	spiTransferByte(0xFF);							// read 16-bit CRC
	spiTransferByte(0xFF);
	CSH;											// release chip select
	return 0;
}
//-------------------------------------------------------------------------
uint8_t mmcWrite(uint32_t sector, uint8_t* buffer){
	uint8_t r1;
	uint16_t i;

	CSL;											// assert chip select	
	r1 = mmcCommand(MMC_WRITE_BLOCK,sector<<9);	// issue command
	if(r1 != 0)return r1;	
	spiTransferByte(0xFF);							// send dummy	
	spiTransferByte(MMC_STARTBLOCK_WRITE);			// send data start token
	
	for(i=0; i<512; i++){
		spiTransferByte(*buffer++);				// write data
	}
	
	spiTransferByte(0xFF);							// write 16-bit CRC (dummy values)
	spiTransferByte(0xFF);
	
	r1 = spiTransferByte(0xFF);						// read data response token
	if((r1&MMC_DR_MASK)!=MMC_DR_ACCEPT)return r1;	
	while(!spiTransferByte(0xFF));					// wait until card not busy
	CSH;											// release chip select
	return 0;
}
//-------------------------------------------------------------------------
uint8_t mmcCommand(uint8_t cmd, uint32_t arg){
	uint8_t r1,retry=0;
	spiTransferByte(cmd|0x40);						// send command
	spiTransferByte(arg>>24);
	spiTransferByte(arg>>16);
	spiTransferByte(arg>>8);
	spiTransferByte(arg);
	//spiTransferByte(0x95);	// crc valid only for MMC_GO_IDLE_STATE
    switch(cmd)
    {
        case CMD_GO_IDLE_STATE:
           spiTransferByte(0x95);
           break;
        case CMD_SEND_IF_COND:
           spiTransferByte(0x87);
           break;
        default:
           spiTransferByte(0xff);
           break;
    }
	// end command
	// wait for response
	// if more than 8 retries, card has timed-out
	// return the received 0xFF
	while((r1=spiTransferByte(0xFF))==0xFF)if(retry++>8)break;
	// return response
	return r1;
}
//-------------------------------------------------------------------------
uint8_t sd_raw_get_info(struct sd_raw_info* info)
{

    memset(info, 0, sizeof(*info));

    
	CSL;
    /* read cid register */
    if(mmcCommand(CMD_SEND_CID, 0))
    {
        CSH;
        return 0;
    }
	
    while(spiTransferByte(0xFF) != 0xfe);
    for(uint8_t i = 0; i < 18; ++i)
    {
        uint8_t b = spiTransferByte(0xFF);

        switch(i)
        {
            case 0:
                info->manufacturer = b;
                break;
            case 1:
            case 2:
                info->oem[i - 1] = b;
                break;
            case 3:
            case 4:
            case 5:
            case 6:
            case 7:
                info->product[i - 3] = b;
                break;
            case 8:
                info->revision = b;
                break;
            case 9:
            case 10:
            case 11:
            case 12:
                info->serial |= (uint32_t) b << ((12 - i) * 8);
                break;
            case 13:
                info->manufacturing_year = b << 4;
                break;
            case 14:
                info->manufacturing_year |= b >> 4;
                info->manufacturing_month = b & 0x0f;
                break;
        }
    }

    /* read csd register */
    uint8_t csd_read_bl_len = 0;
    uint8_t csd_c_size_mult = 0;
#if SD_RAW_SDHC
    uint16_t csd_c_size = 0;
#else
    uint32_t csd_c_size = 0;
#endif
    if(mmcCommand(CMD_SEND_CSD, 0))
    {
        CSH;
        return 0;
    }
    while(spiTransferByte(0xFF) != 0xfe);
    for(uint8_t i = 0; i < 18; ++i)
    {
        uint8_t b = spiTransferByte(0xFF);

        if(i == 14)
        {
            if(b & 0x40)
                info->flag_copy = 1;
            if(b & 0x20)
                info->flag_write_protect = 1;
            if(b & 0x10)
                info->flag_write_protect_temp = 1;
            info->format = (b & 0x0c) >> 2;
        }
        else
        {
#if SD_RAW_SDHC
            if(sd_raw_card_type & (1 << SD_RAW_SPEC_2))
            {
                switch(i)
                {
                    case 7:
                        b &= 0x3f;
                    case 8:
                    case 9:
                        csd_c_size <<= 8;
                        csd_c_size |= b;
                        break;
                }
                if(i == 9)
                {
                    ++csd_c_size;
                    //info->capacity = (offset_t) csd_c_size * 512 * 1024;//bytes
					info->capacity = (offset_t) csd_c_size /2;
                }
            }
            else
#endif
            {
                switch(i)
                {
                    case 5:
                        csd_read_bl_len = b & 0x0f;
                        break;
                    case 6:
                        csd_c_size = b & 0x03;
                        csd_c_size <<= 8;
                        break;
                    case 7:
                        csd_c_size |= b;
                        csd_c_size <<= 2;
                        break;
                    case 8:
                        csd_c_size |= b >> 6;
                        ++csd_c_size;
                        break;
                    case 9:
                        csd_c_size_mult = b & 0x03;
                        csd_c_size_mult <<= 1;
                        break;
                    case 10:
                        csd_c_size_mult |= b >> 7;

                        info->capacity = (uint32_t) csd_c_size << (csd_c_size_mult + csd_read_bl_len + 2);
						info->capacity/=1024; //KB
						info->capacity/=1024; //MB
                        break;
                }
            }
        }
    }

    CSH;

    return 1;
}
//-------------------------------------------------------------------------
