/*
+------------------------------------------------------------------------------
| Project   : LED_KEY Test for Xmega3_DEMO
+------------------------------------------------------------------------------
| Copyright 2010  www.chiplab7.com (lisn3188@163.com)
| All rights reserved.
|------------------------------------------------------------------------------
| File      : KEY_LED.c, IO operation demo 
|------------------------------------------------------------------------------
| Chang Logs:
| Date           Author           Notes
| 2010-06-22     lisn3188         The first version.                     
+------------------------------------------------------------------------------
*/
#include <avr/io.h>

//---------led operation--------
#define LED1_ON()   PORTC.OUT&=~0x01
#define LED1_OFF()  PORTC.OUT|=0x01
#define LED1_T()    PORTC.OUT^=0x01

#define LED2_ON()   PORTC.OUT&=~0x02
#define LED2_OFF()  PORTC.OUT|=0x02
#define LED2_T()    PORTC.OUT^=0x02


//--------key return vaule-------
#define No_key   0x00
#define SELECT   0x01
#define LEFT     0x02
#define RIGHT    0x04
#define UP       0x08
#define DOWN     0x10

/*
+------------------------------------------------------------------------------
| Function    : KEY_initial
+------------------------------------------------------------------------------
| Description : set the Key Pin as input with pull up resistance
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void KEY_initial(void)
{
	PORTE.DIR&=~0x1f;
	PORTE.PIN0CTRL=PORT_OPC_PULLUP_gc;
	PORTE.PIN1CTRL=PORT_OPC_PULLUP_gc;
	PORTE.PIN2CTRL=PORT_OPC_PULLUP_gc;
	PORTE.PIN3CTRL=PORT_OPC_PULLUP_gc;
	PORTE.PIN4CTRL=PORT_OPC_PULLUP_gc;
}

/*
+------------------------------------------------------------------------------
| Function    : Get_Key
+------------------------------------------------------------------------------
| Description : Test the key
|
| Parameters  : 
| Returns     : pressing key(if more than one key is pressed return 0 )
|
+------------------------------------------------------------------------------
*/
unsigned char Get_Key(void)
{
	unsigned char Key=0,num_keypress=0;
	if((PORTE.IN&(1<<1))==0){Key|=SELECT;num_keypress++;}
	if((PORTE.IN&(1<<0))==0){Key|=LEFT;num_keypress++;}
	if((PORTE.IN&(1<<4))==0){Key|=RIGHT;num_keypress++;}
	if((PORTE.IN&(1<<2))==0){Key|=UP;num_keypress++;}
	if((PORTE.IN&(1<<3))==0){Key|=DOWN;num_keypress++;}
	if(num_keypress>1)Key=No_key;
	return Key;
}

/*
+------------------------------------------------------------------------------
| Function    : main
+------------------------------------------------------------------------------
| Description : trun on the LED
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
int main(void)
{
unsigned char Key_return=0;
PORTC.DIR|=0x03; //Set Led Pin as Output
LED1_ON(); //trun off the LED
LED2_ON(); //trun off the LED

KEY_initial();//initial the Key Pin
while(1){
	Key_return=Get_Key(); //get the pressing key
if(Key_return){
	switch(Key_return){
		case SELECT: LED1_T();LED2_T();break;  
		case LEFT  : LED1_ON();break;
		case RIGHT : LED1_OFF();break;
		case UP    : LED2_ON();break;
		case DOWN  : LED2_OFF();break;
		default :break;
	}
	Key_return=0;
}
}
return 0;
}















