/*************************************************************************
 *
 *
 *              Task1: Basic I/O
 *
 *
 *************************************************************************/

// The avr_compiler.h header file includes the MCU specific header 
// file configured in the project options and it also contains 
// compatibility definitions to be able to compile the same code 
// for both the GCC and the IAR compiler. 

// For GCC:
// This will include avr/iox128a1.h for us, since the
// MCU = atxmega128a1 is defined in the project make file.
#include <avr/io.h>

// For IAR:
// This will include ioxm128a1.h, since target is set to
// --cpu=xm128a1 in the project configuration.

// In addition it contains some macros to make the code more 
// similar for both compilers. Look into the file for more details.
//#include "avr_compiler.h"

// Since all IO port access refers to a PORT struct defined in the header
// file, it is enough to define the module name only, instead of creating
// #defines for all registers for the port module. This #define selects the
// IO port to use for LED output in this demo application.
void Delay()
{
	volatile unsigned int counter;
	for ( counter=0; counter<50000; counter++) 
		{
			;
		}
}


int main( void )
{
   unsigned char num = 0x01;
   PORTF.DIR = 0xff;    // Set all pins of port D to output.
   PORTF.OUT = 0x00;    // Set upper four bits high.
   Delay();
   PORTF.OUTSET = 0x0F;
   Delay();
   PORTF.OUTCLR = 0x0F;
   Delay();
   PORTF.OUTTGL = 0XFF;
   Delay();
   PORTF.OUTTGL = 0XFF;
   Delay();
   PORTF.OUTTGL = 0XFF;
   Delay();
   PORTF.OUTTGL = 0XAA;
   Delay();
   PORTF.OUTTGL = 0XAA;
   Delay();
   PORTF.OUTTGL = 0XAA;
   Delay();
   
   // Now, just for fun, we toggle all LEDs, again and again and again...
   for (;;)
   {
        PORTF.OUT = num ;
        Delay();
		num = num << 1;
		if (num == 0)
			num = 0x01;
	
		
   }
}
