
/*
 * Copyright (c) 2006-2010 by Roland Riegel <feedback@roland-riegel.de>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/sfr_defs.h>
#include <avr/sleep.h>
//-----------------------------------------------------------
#include "uart.h"
//ʹUARTE1
#define USART USARTE1

#define BAUD 115200UL
#define USE_SLEEP 0
//-----------------------------------------------------------
void uart_init()
{
/* PIN7 (TXD0) as output. */
	PORTE.DIRSET = PIN7_bm;
	/* PE6 (RXD0) as input. */
	PORTE.DIRCLR = PIN6_bm;
	/* USARTC0, 8 Data bits, No Parity, 1 Stop bit. */
	USART.CTRLC = USART_CHSIZE_8BIT_gc;
	/* Set Baudrate to 115200 bps:
	 * Use the default I/O clock fequency that is 32 MHz.
	 * Do not use the baudrate scale factor
	 *
	 */
	USART.BAUDCTRLA = 0x17;  //115200
	USART.BAUDCTRLB = 0xa4;
	/* Enable both RX and TX. */
	USART.CTRLB =USART_RXEN_bm | USART_TXEN_bm;
}
//-----------------------------------------------------------
void uart_putc(uint8_t c)
{
    if(c == '\n')
        uart_putc('\r');

    /* wait until transmit buffer is empty */
    while(!(USART.STATUS & USART_DREIF_bm));

    /* send next byte */
    USART.DATA = c;
}
//-----------------------------------------------------------
void uart_putc_hex(uint8_t b)
{
    /* upper nibble */
    if((b >> 4) < 0x0a)
        uart_putc((b >> 4) + '0');
    else
        uart_putc((b >> 4) - 0x0a + 'a');

    /* lower nibble */
    if((b & 0x0f) < 0x0a)
        uart_putc((b & 0x0f) + '0');
    else
        uart_putc((b & 0x0f) - 0x0a + 'a');
}
//-----------------------------------------------------------
void uart_putw_hex(uint16_t w)
{
    uart_putc_hex((uint8_t) (w >> 8));
    uart_putc_hex((uint8_t) (w & 0xff));
}

void uart_putdw_hex(uint32_t dw)
{
    uart_putw_hex((uint16_t) (dw >> 16));
    uart_putw_hex((uint16_t) (dw & 0xffff));
}
//-----------------------------------------------------------
void uart_putw_dec(uint16_t w)
{
    uint16_t num = 10000;
    uint8_t started = 0;

    while(num > 0)
    {
        uint8_t b = w / num;
        if(b > 0 || started || num == 1)
        {
            uart_putc('0' + b);
            started = 1;
        }
        w -= b * num;

        num /= 10;
    }
}
//-----------------------------------------------------------
void uart_putdw_dec(uint32_t dw)
{
    uint32_t num = 1000000000;
    uint8_t started = 0;

    while(num > 0)
    {
        uint8_t b = dw / num;
        if(b > 0 || started || num == 1)
        {
            uart_putc('0' + b);
            started = 1;
        }
        dw -= b * num;

        num /= 10;
    }
}
//-----------------------------------------------------------
void uart_puts(const char* str)
{
    while(*str)
        uart_putc(*str++);
}
//-----------------------------------------------------------
void uart_puts_p(PGM_P str)
{
    while(1)
    {
        uint8_t b = pgm_read_byte_near(str++);
        if(!b)
            break;

        uart_putc(b);
    }
}
//-----------------------------------------------------------
uint8_t uart_getc()
{
    /* wait until receive buffer is full */
#if USE_SLEEP
    uint8_t sreg = SREG;
    sei();

    while(!(UCSRA & (1 << RXC)))
        sleep_mode();

    SREG = sreg;
#else
    while(!(USART.STATUS & USART_RXCIF_bm));
#endif

    uint8_t b = USART.DATA;
    if(b == '\r')
        b = '\n';

    return b;
}
//-----------------------------------------------------------

