/*
+------------------------------------------------------------------------------
| Project   : ADC Test for Xmega3_DEMO
+------------------------------------------------------------------------------
| Copyright 2010  www.chiplab7.com (lisn3188@163.com)
| All rights reserved.
|------------------------------------------------------------------------------
| File      : ADC.c, ADC operation demo 
|------------------------------------------------------------------------------
| Chang Logs:
| Date           Author           Notes
| 2010-06-22     lisn3188         The first version.                     
+------------------------------------------------------------------------------
*/
#include <avr/io.h>
#include "OLED.h"

/*
+------------------------------------------------------------------------------
| Function    : Initial_ADC
+------------------------------------------------------------------------------
| Description : initail the ADC model
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void Initial_ADC(void)
{
ADCA.CTRLB=ADC_RESOLUTION_12BIT_gc;  /* 12-bit right-adjusted result */
ADCA.PRESCALER=ADC_PRESCALER_DIV32_gc;/* Divide clock by 32 */
ADCA.REFCTRL=ADC_REFSEL_VCC_gc; /* Internal VCC / 1.6V */
ADCA.CH0.CTRL=ADC_CH_INPUTMODE_SINGLEENDED_gc|ADC_CH_GAIN_1X_gc;/* Single-ended input, no gain */
ADCA.CH1.CTRL=ADC_CH_INPUTMODE_SINGLEENDED_gc|ADC_CH_GAIN_1X_gc;/* Single-ended input, no gain */

ADCA.CH0.MUXCTRL=ADC_CH_MUXPOS_PIN0_gc;/* Input pin 0 */
ADCA.CH1.MUXCTRL=ADC_CH_MUXPOS_PIN1_gc;/* Input pin 1 */
ADCA.EVCTRL |= ADC_SWEEP_01_gc; /* Channel sweep ADC Channel 0,1 */
ADCA.CTRLA |= ADC_ENABLE_bm; /* Enable ADC */
ADCA.CTRLB |= ADC_FREERUN_bm; /* Free Running Mode Enable */

}

/*
+------------------------------------------------------------------------------
| Function    : Read_ADC
+------------------------------------------------------------------------------
| Description : Read ADC value
|
| Parameters  : adc_ch  channel to read
| Returns     : ADC convert vaule
|
+------------------------------------------------------------------------------
*/
int Read_ADC(unsigned char adc_ch) //read ch1 or ch0;
{
if(adc_ch==0){
	  	do{
			/* If the conversion on the ADCA channel 0 never is
			 * complete this will be a deadlock. */
		}while(((ADCA.CH0.INTFLAGS & ADC_CH_CHIF_bm) == 0x00));
ADCA.CH0.INTFLAGS |= ADC_CH_CHIF_bm;

return ADCA.CH0.RES;
}
else if(adc_ch==1){
	  	do{
			/* If the conversion on the ADCA channel 0 never is
			 * complete this will be a deadlock. */
		}while(((ADCA.CH1.INTFLAGS & ADC_CH_CHIF_bm) == 0x00));
ADCA.CH1.INTFLAGS |= ADC_CH_CHIF_bm;

return ADCA.CH1.RES;
}
else return 0;
}

/*
+------------------------------------------------------------------------------
| Function    : main
+------------------------------------------------------------------------------
| Description : trun on the LED
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/

int main(void)
{
unsigned int ADC_CH0,ADC_CH1;

OLED_Port_initial();
OLED_initial();
OLED_Clear();
Initial_ADC();
display_list_char(0,2,(void*)"  XMEGA_ADC-TEST");

while(1){
ADC_CH0=(unsigned int)Read_ADC(0);
ADC_CH1=(unsigned int)Read_ADC(1);
Display_num(1,6,ADC_CH0);
Display_Bar(38,6,(unsigned char)(ADC_CH0>>3));

Display_num(1,7,ADC_CH1);
Display_Bar(38,7,(unsigned char)(ADC_CH1>>3));

}

return 0;
}















