/*
+------------------------------------------------------------------------------
| Project   : Clock systems Test for Xmega3_DEMO
+------------------------------------------------------------------------------
| Copyright 2010  www.chiplab7.com (lisn3188@163.com)
| All rights reserved.
|------------------------------------------------------------------------------
| File      : Clock.c, Clock operation demo 
|------------------------------------------------------------------------------
| Chang Logs:
| Date           Author           Notes
| 2010-06-22     lisn3188         The first version.                     
+------------------------------------------------------------------------------
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "OLED.h"

//--------key return vaule-------
#define No_key   0x00
#define SELECT   0x01
#define LEFT     0x02
#define RIGHT    0x04
#define UP       0x08
#define DOWN     0x10

#define LED1_ON()   PORTC.OUT&=~0x01
#define LED1_OFF()  PORTC.OUT|=0x01
#define LED1_T()    PORTC.OUT^=0x01

#define LED2_ON()   PORTC.OUT&=~0x02
#define LED2_OFF()  PORTC.OUT|=0x02
#define LED2_T()    PORTC.OUT^=0x02

/*
+------------------------------------------------------------------------------
| Function    : KEY_initial
+------------------------------------------------------------------------------
| Description : set the Key Pin as input with pull up resistance
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void KEY_initial(void)
{
	PORTE.DIR&=~0x1f;
	PORTE.PIN0CTRL=PORT_OPC_PULLUP_gc;
	PORTE.PIN1CTRL=PORT_OPC_PULLUP_gc;
	PORTE.PIN2CTRL=PORT_OPC_PULLUP_gc;
	PORTE.PIN3CTRL=PORT_OPC_PULLUP_gc;
	PORTE.PIN4CTRL=PORT_OPC_PULLUP_gc;
}

/*
+------------------------------------------------------------------------------
| Function    : Get_Key
+------------------------------------------------------------------------------
| Description : Test the key
|
| Parameters  : 
| Returns     : pressing key(if more than one key is pressed return 0 )
|
+------------------------------------------------------------------------------
*/
unsigned char Get_Key(void)
{
	unsigned char Key=0,num_keypress=0;
	if((PORTE.IN&(1<<1))==0){Key|=SELECT;num_keypress++;}
	if((PORTE.IN&(1<<0))==0){Key|=LEFT;num_keypress++;}
	if((PORTE.IN&(1<<4))==0){Key|=RIGHT;num_keypress++;}
	if((PORTE.IN&(1<<2))==0){Key|=UP;num_keypress++;}
	if((PORTE.IN&(1<<3))==0){Key|=DOWN;num_keypress++;}
	if(num_keypress>1)Key=No_key;
	return Key;
}

/*
+------------------------------------------------------------------------------
| Function    : Initial_Timer
+------------------------------------------------------------------------------
| Description : initial the Timer 
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void Initial_Timer(void)
{
TCC1.PER=976*2;  //
TCC1.INTCTRLA=TC_OVFINTLVL_LO_gc;//Enable interrupt
/* Select clock source. 1024Div */
TCC1.CTRLA=TC_CLKSEL_DIV1024_gc;
PMIC.CTRL |= PMIC_LOLVLEN_bm;
}

/*
+------------------------------------------------------------------------------
| Function    : OSC_8M_PLL_SystemsClk
+------------------------------------------------------------------------------
| Description : ext OSC 8M use PLL  
|  				16M CLK
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void OSC_8M_PLL_SystemsClk(void)
{

OSC.CTRL &=~OSC_RC32MEN_bm; //Disable RC32M
//Setting Ext. OSC//XTAL_16KCLK 0.4 - 16 MHz XTAL 16K CLK
OSC.XOSCCTRL=OSC_FRQRANGE_2TO9_gc | OSC_X32KLPM_bm | OSC_XOSCSEL_XTAL_16KCLK_gc;
/* Turn on Ext. 8M & 32kHz. */
OSC.CTRL |=OSC_XOSCEN_bm;
	do {
		/* Wait for the oscillators to stabilize. */
	} while ( ( OSC.STATUS & OSC_XOSCRDY_bm ) == 0);

OSC.PLLCTRL= (uint8_t)OSC_PLLSRC_XOSC_gc | 0x2; //8M* 4=16M 
OSC.CTRL|=OSC_PLLEN_bm;  //Enable PLL
CLK.PSCTRL= (uint8_t)CLK_PSADIV_1_gc | CLK_PSBCDIV_1_1_gc;//prescaler configuration
while((OSC.STATUS & OSC_PLLRDY_bm)==0); //Wait PLL Ready
CCP = CCP_IOREG_gc;
CLK.CTRL = CLK_SCLKSEL_PLL_gc;   //Use PLL as Main Clock

}

/*
+------------------------------------------------------------------------------
| Function    : Int_32M_RC_SystemsClk
+------------------------------------------------------------------------------
| Description : int 32M RC 
|  				32M CLK
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void Int_32M_RC_SystemsClk(void)
{
		/*  Enable internal 32 MHz ring oscillator and wait until it's
		 *  stable. 
		 *  32 MHz ring oscillator as the main clock source. 
		 */
		OSC.CTRL &=~OSC_XOSCEN_bm; //Disable ext OSC
		OSC.CTRL &=~OSC_PLLEN_bm; //Disable PLL
		OSC.CTRL |= OSC_RC32MEN_bm;
		CLK.PSCTRL= (uint8_t)CLK_PSADIV_1_gc | CLK_PSBCDIV_1_1_gc;//prescaler configuration
		while((OSC.STATUS & OSC_RC32MRDY_bm)==0); //Wait 32M RC Ready
		CCP = CCP_IOREG_gc;
		CLK.CTRL = CLK_SCLKSEL_RC32M_gc;   //Use 32M RC as Main Clock
}

/*
+------------------------------------------------------------------------------
| Function    : Int_2M_RC_SystemsClk
+------------------------------------------------------------------------------
| Description : int 2M RC 
|  				2M CLK
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void Int_2M_RC_SystemsClk(void)
{

		OSC.CTRL &=~OSC_XOSCEN_bm; //Disable ext OSC
		OSC.CTRL &=~OSC_PLLEN_bm; //Disable PLL
		OSC.CTRL &=~OSC_RC32MEN_bm; //Disable RC32M
		CLK.PSCTRL= (uint8_t)CLK_PSADIV_1_gc | CLK_PSBCDIV_1_1_gc;//prescaler configuration
		while((OSC.STATUS & OSC_RC2MRDY_bm)==0); //Wait 2M RC Ready
		CCP = CCP_IOREG_gc;
		CLK.CTRL = CLK_SCLKSEL_RC2M_gc;   //Use 32M RC as Main Clock
}

/*
+------------------------------------------------------------------------------
| Function    : Wait_Key_Press
+------------------------------------------------------------------------------
| Description : Wait Key Press ,
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void Wait_Key_Press(void)
{
while(Get_Key()==No_key);
while(Get_Key()!=No_key);
}
/*
+------------------------------------------------------------------------------
| Function    : main
+------------------------------------------------------------------------------
| Description : sweet the CPU CLK source
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
int main(void)
{
KEY_initial();
PORTC.DIR|=0x03; //Set Led Pin as Output
LED1_OFF();
LED2_OFF();
OLED_Port_initial();
OLED_initial();
OLED_Clear();
display_list_char(0,2,(void*)"  XMEGA_Clock-TEST");
display_list_char(0,5,(void*)" Int_2M_RC(2Mhz)");
display_list_char(0,6,(void*)" OSC_8M_PLL(16Mhz)");
display_list_char(0,7,(void*)" Int_32M_RC(32Mhz)");
Initial_Timer();
sei();
while(1){ 
Int_2M_RC_SystemsClk(); //2M
display_one_char(0,7,' ');
display_one_char(0,5,'>');

Wait_Key_Press(); //Wait Key Press 

OSC_8M_PLL_SystemsClk(); //16M
display_one_char(0,5,' ');
display_one_char(0,6,'>');

Wait_Key_Press();//Wait Key Press 

Int_32M_RC_SystemsClk(); //32M
display_one_char(0,6,' ');
display_one_char(0,7,'>');
Wait_Key_Press();//Wait Key Press 

}

return 0;
}



/*
+------------------------------------------------------------------------------
| Function    : interrput 
+------------------------------------------------------------------------------
| Description : interrput for TCC1 overflow
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
ISR(TCC1_OVF_vect)  //
{
LED1_T();
}






