/*
+------------------------------------------------------------------------------
| Project   : OLED Test for Xmega3_DEMO
+------------------------------------------------------------------------------
| Copyright 2010  www.chiplab7.com (lisn3188@163.com)
| All rights reserved.
|------------------------------------------------------------------------------
| File      : OLED.c, OLED display demo 
|------------------------------------------------------------------------------
| Chang Logs:
| Date           Author           Notes
| 2010-06-22     lisn3188         The first version.                     
+------------------------------------------------------------------------------
*/
#include <avr/io.h>
#include "Pixel_EN.h"
//--------------------------------------------------------------------
#define NOP()	asm("nop")
#define Longnop() NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();
//--------------------------------------------------------------------
/*
CS   -> PD0
REST -> PD1
D/C  -> PD2
DATA -> PD5
CLK  -> PD3
*/
#define CS_H()   PORTD.OUT|=0x01
#define CS_L()   PORTD.OUT&=~0x01

#define RES_H()  PORTD.OUT|=0x02
#define RES_L()  PORTD.OUT&=~0x02

#define DC_Com() PORTD.OUT&=~0x04
#define DC_Dat() PORTD.OUT|=0x04

#define CLK_H()  PORTD.OUT|=0x08
#define CLK_L()  PORTD.OUT&=~0x08

#define Dat_H()  PORTD.OUT|=0x20
#define Dat_L()  PORTD.OUT&=~0x20

/*
+------------------------------------------------------------------------------
| Function    : OLED_Port_initial
+------------------------------------------------------------------------------
| Description : initial OLED contrl Pin as Output
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void OLED_Port_initial(void)
{
PORTD.DIR=0xAF;
PORTD.OUT|=0x1f;
}

/*
+------------------------------------------------------------------------------
| Function    : OLED_Send_Comm
+------------------------------------------------------------------------------
| Description : send a 8bit command to OLED use serial mode
|
| Parameters  : comm  command to send
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void OLED_Send_Comm(unsigned char comm)
{
unsigned char i;
DC_Com();
CS_L();
for(i=0;i<8;i++)
{
CLK_L();
if(comm&0x80)Dat_H();
	else Dat_L();
CLK_H();
comm<<=1;
}
CS_H();
}

/*
+------------------------------------------------------------------------------
| Function    : OLED_Send_Data
+------------------------------------------------------------------------------
| Description : send 8bits data to OLED use serial mode
|
| Parameters  : Data  data to send 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void OLED_Send_Data(unsigned char Data)
{
unsigned char i;
DC_Dat();
CS_L();
for(i=0;i<8;i++)
{
CLK_L();
if(Data&0x80)Dat_H();
	else Dat_L();
CLK_H();
Data<<=1;
}
CS_H();
}

/*
+------------------------------------------------------------------------------
| Function    : OLED_initial
+------------------------------------------------------------------------------
| Description : initial the OLED control registers ,let it ready to use
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void OLED_initial(void)
{
RES_L();
NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();
RES_H();
//Charge Pump Setting
OLED_Send_Comm(0x8d);
OLED_Send_Comm(0x14);
OLED_Send_Comm(0xaf);
//Set Display Clock 
OLED_Send_Comm(0xD5);
OLED_Send_Comm(0xF0);
//Set Pre-charge Period
OLED_Send_Comm(0xD9);
OLED_Send_Comm(0x11);
//Set VCOMH Deselect Level 
OLED_Send_Comm(0xDb);
OLED_Send_Comm(0x0);
//Set Norma Display 
OLED_Send_Comm(0xa6);
//Entire Display ON
//OLED_Send_Comm(0xa5);
//Set Contrast Control
OLED_Send_Comm(0x81);
OLED_Send_Comm(0xff);  //1-256 
//Set Segment Re-map
OLED_Send_Comm(0xa1);
//Set COM Output Scan Direction 
OLED_Send_Comm(0xc8);
//Set Memory Addressing Mode 
OLED_Send_Comm(0x20);
OLED_Send_Comm(0x00);
//Set Fade Out and Blinking
//OLED_Send_Comm(0x23);
//OLED_Send_Comm(0x30);
}

/*
+------------------------------------------------------------------------------
| Function    : OLED_GOTO
+------------------------------------------------------------------------------
| Description : set the cursor address
|
| Parameters  : x  horizontal (0-127)
				y  vertical (Page 0-7)
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void OLED_GOTO(unsigned char x,unsigned char y)
{
//Set Page Address 
OLED_Send_Comm(0x22);
OLED_Send_Comm(y);  // start 
OLED_Send_Comm(7);  // end
//Set Column Address
OLED_Send_Comm(0x21);
OLED_Send_Comm(x);  // start 
OLED_Send_Comm(127);  // end
}

/*
+------------------------------------------------------------------------------
| Function    : OLED_Clear
+------------------------------------------------------------------------------
| Description : clear the OLED buffer ,turn off the all display point.
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void OLED_Clear(void)
{
unsigned int i;
OLED_GOTO(0,0);
for(i=0;i<1024;i++)OLED_Send_Data(0x0);
}

/*
+------------------------------------------------------------------------------
| Function    : display_one_char
+------------------------------------------------------------------------------
| Description : Display one char at specify address
|
| Parameters  : x horizontal (0-21)
				y vertical (Page 0-7)
				char_data char to display (in ASCII)
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void display_one_char(unsigned char x,unsigned char y,unsigned char char_data)
{
unsigned char i;
OLED_GOTO(x*6,y);
char_data-=32;
for (i=0;i<6;i++)
{

OLED_Send_Data(Font6x8[char_data][i]); 

}
OLED_Send_Data(0);

}

/*
+------------------------------------------------------------------------------
| Function    : display_list_char
+------------------------------------------------------------------------------
| Description : Display one char at specify address
|
| Parameters  : x horizontal (0-21)
				y vertical (Page 0-7)
				*data_s  Pointer to string to display (in ASCII)
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void display_list_char(unsigned char x,unsigned char y,unsigned char *data_s)
{
  while(*data_s) 
  {
    display_one_char(x, y, *data_s); //ʾַ
     x++;
	 data_s++;
   }
}

/*
+------------------------------------------------------------------------------
| Function    : Display_num
+------------------------------------------------------------------------------
| Description : Display a number in decimal(0-99999)
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void Display_num(unsigned char x,unsigned char y,unsigned int num)
{
display_one_char(x++, y,num/10000+0x30);
display_one_char(x++, y,num%10000/1000+0x30);
display_one_char(x++, y,num%1000/100+0x30);
display_one_char(x++, y,num%100/10+0x30);
display_one_char(x++, y,num%10+0x30);
}

/*
+------------------------------------------------------------------------------
| Function    : Display_Bar
+------------------------------------------------------------------------------
| Description : 
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void Display_Bar(unsigned char x,unsigned char y,unsigned char num)
{
unsigned char i,j;
i=num/13;  //0-19
OLED_GOTO(x,y);
OLED_Send_Data(0x7e);
OLED_Send_Data(0x81);
for(j=0;j<i;j++)
{
OLED_Send_Data(0xbd);
OLED_Send_Data(0xbd);
OLED_Send_Data(0xbd);
OLED_Send_Data(0x81);
}
for(j=0;j<19-i;j++)
{
OLED_Send_Data(0x81);
OLED_Send_Data(0x81);
OLED_Send_Data(0x81);
OLED_Send_Data(0x81);
}
OLED_Send_Data(0x7e);
}
//--------------------------------------------------------------------
//End Of The File
