/*
+------------------------------------------------------------------------------
| Project   : RTC Test for Xmega3_DEMO
+------------------------------------------------------------------------------
| Copyright 2010  www.chiplab7.com (lisn3188@163.com)
| All rights reserved.
|------------------------------------------------------------------------------
| File      : RTC.c, RTC operation demo 
|------------------------------------------------------------------------------
| Chang Logs:
| Date           Author           Notes
| 2010-06-22     lisn3188         The first version.                     
+------------------------------------------------------------------------------
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "OLED.h"

#define LED1_ON()   PORTC.OUT&=~0x01
#define LED1_OFF()  PORTC.OUT|=0x01
#define LED1_T()    PORTC.OUT^=0x01

#define LED2_ON()   PORTC.OUT&=~0x02
#define LED2_OFF()  PORTC.OUT|=0x02
#define LED2_T()    PORTC.OUT^=0x02


volatile unsigned int System_Timer=0;
volatile unsigned char RTC_overflow_flag=0;

/*
+------------------------------------------------------------------------------
| Function    : Initial_RTC
+------------------------------------------------------------------------------
| Description : Initial RTC ,Use internal 32kHz ULP for main clk
|				PER=1024 ,1S Overflow
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/
void Initial_RTC(void)
{
//Enable RTC
CLK.RTCCTRL = CLK_RTCSRC_ULP_gc | CLK_RTCEN_bm;	/* 1kHz from internal 32kHz ULP */
while(RTC.STATUS & RTC_SYNCBUSY_bm); //wait clk ready
RTC.PER=1024;  //per 1S  a interrupt
RTC.CNT=0;
RTC.COMP=0;
RTC.CTRL=RTC_PRESCALER_DIV1_gc;
RTC.INTCTRL=RTC_OVFINTLVL_LO_gc;
/* Enable interrupts. */
PMIC.CTRL |= PMIC_LOLVLEN_bm;
sei();
}

/*
+------------------------------------------------------------------------------
| Function    : main
+------------------------------------------------------------------------------
| Description : trun on the LED
|
| Parameters  : 
| Returns     : 
|
+------------------------------------------------------------------------------
*/

int main(void)
{

PORTC.DIR|=0x03; //Set Led Pin as Output
LED1_OFF(); //trun off the LED
LED2_OFF(); //trun off the LED
OLED_Port_initial();
OLED_initial();
OLED_Clear();
display_list_char(0,2,(void*)"  XMEGA_RTC-TEST");
display_list_char(0,5,(void*)"RTC-Count:00000-Sec");
Initial_RTC();
while(1){
if(RTC_overflow_flag){
	Display_num(10,5,System_Timer);
	RTC_overflow_flag=0;
	}
}
return 0;
}


//------------------------------------------------------------
ISR(RTC_OVF_vect)
{
LED1_T(); //RTC interrput
RTC_overflow_flag=1;
System_Timer++;
if(System_Timer>99999)System_Timer=0;
}








