
#include "avr_compiler.h"
#include "eeprom_driver.h"

#define TEST_BYTE_1 0x5d
#define TEST_BYTE_2 0xAb

#define TEST_BYTE_ADDR_1 0x01
#define TEST_BYTE_ADDR_2 0x06

#define TEST_PAGE_ADDR_1    0  /* Page address always on a page boundary. */
#define TEST_PAGE_ADDR_2    2  /* Page address always on a page boundary. */
#define TEST_PAGE_ADDR_3    5  /* Page address always on a page boundary. */
//--------------------------------------------------------------------------
#define LED1_ON()   PORTC.OUT&=~0x01
#define LED1_OFF()  PORTC.OUT|=0x01
#define LED1_T()    PORTC.OUT^=0x01

#define LED2_ON()   PORTC.OUT&=~0x02
#define LED2_OFF()  PORTC.OUT|=0x02
#define LED2_T()    PORTC.OUT^=0x02


//--------------------------------------------------------------------------

/*! \brief Example code writing and reading the EEPROM with different methods.
 *
 *   This example code show how to read and write the EEPROM while it is both
 *   memory mapped and IO mapped.
 *
 */
int main( void )
{
	/* Assume OK from the start. Set to 0 on first error.*/
	bool test_ok = true;
	
	PORTC.DIR|=0x03;
	LED1_ON();
	LED2_OFF();
	
	/* Flush buffer just to be sure when we start.*/
	EEPROM_FlushBuffer();

	/* ***
	 * Write and read two bytes using IO-mapped access.
	 * ***/

	EEPROM_DisableMapping();

	/* Write bytes. */
	EEPROM_WriteByte(TEST_PAGE_ADDR_1, TEST_BYTE_ADDR_1, TEST_BYTE_1);
	EEPROM_WriteByte(TEST_PAGE_ADDR_1, TEST_BYTE_ADDR_2, TEST_BYTE_2);

	/* Read back and check bytes.*/
	if (EEPROM_ReadByte(TEST_PAGE_ADDR_1, TEST_BYTE_ADDR_1) != TEST_BYTE_1) {
		test_ok = false;
	}

	if (EEPROM_ReadByte(TEST_PAGE_ADDR_1, TEST_BYTE_ADDR_2) != TEST_BYTE_2) {
		test_ok = false;
	}
	
	
	/* ***
	 * Now, check if everything went well.
	 * ***/
	if (true == test_ok) {
		LED2_ON();
		while(1) {
			
			/* Success. */
			nop();
		}
	} else {
		while(1) {
			/* Failure. */
			nop();
		}
	}
	
return 0;
}



ISR(NVM_EE_vect) 
{ 
    

   /* Disable the EEPROM interrupt */ 
   NVM.INTCTRL = (NVM.INTCTRL & ~NVM_EELVL_gm); 
    
   /* only for test */ 
   /* turn led off  */ 
   //   LED_AUS 
} 